var hbuilder = require("hbuilderx")
var fs = require("fs")
var manifestUtil = require("./manifest")
var projectResources = require("./projectresources")
var util = require("./util")
var projectutil = require("./projectutil")
var hjson = require("./hjson.min")
var AdmZip = require("adm-zip");
var os = require("os");
var path = require("path");

function containsIos(params) {
    if (!params) return false;

    if (params.indexOf("IOS-APPSTORE") > -1) {
        return true;
    }
    return false;
}

function copyImageFiles(project, manifastDir, imageObject, imageJson, prefix) {
    var ret = {};
    try {
        for (var field in imageObject) {
            var fieldValue = imageObject[field];
            if (!util.isStringEmpty(fieldValue)) {
                var fileName = util.copyFile(project, fieldValue, prefix + field, manifastDir);
                if (fileName) {
                    ret[field] = fileName;
                }
            }
        }
        return ret;
    } catch (e) {
        console.error(e);
    }
    return;
}

function containsAndroid(params) {
    if (!params) return false;

    if (params.indexOf("ANDROID-GOOGLEPLAY") > -1 ||
        params.indexOf("ANDROID-PB") > -1 ||
        params.indexOf("ANDROID-CLC") > -1 ||
        params.indexOf("ANDROID-TEST") > -1) {
        return true;
    }
    return false;
}

function getadsPermissions() {
    var permission = [
        "<uses-permission android:name=\"android.permission.RECORD_AUDIO\"/>",
        "<uses-permission android:name=\"android.permission.MODIFY_AUDIO_SETTINGS\"/>",
        "<uses-permission android:name=\"android.permission.CAMERA\"/>",
        "<uses-permission android:name=\"android.permission.FLASHLIGHT\"/>",
        "<uses-feature android:name=\"android.hardware.camera.autofocus\"/>",
        "<uses-permission android:name=\"android.permission.WRITE_EXTERNAL_STORAGE\"/>",
        "<uses-feature android:name=\"android.hardware.camera\"/>",
        "<uses-permission android:name=\"android.permission.WAKE_LOCK\"/>",
        "<uses-permission android:name=\"android.permission.VIBRATE\"/>",
        "<uses-permission android:name=\"android.permission.CALL_PHONE\"/>",
        "<uses-permission android:name=\"android.permission.INTERNET\"/>",
        "<uses-permission android:name=\"android.permission.ACCESS_COARSE_LOCATION\"/>",
        "<uses-permission android:name=\"android.permission.ACCESS_FINE_LOCATION\"/>",
        "<uses-permission android:name=\"android.permission.ACCESS_WIFI_STATE\"/>",
        "<uses-permission android:name=\"android.permission.ACCESS_NETWORK_STATE\"/>",
        "<uses-permission android:name=\"android.permission.CHANGE_WIFI_STATE\"/>",
        "<uses-permission android:name=\"android.permission.READ_PHONE_STATE\"/>",
        "<uses-permission android:name=\"android.permission.READ_LOGS\"/>",
        "<uses-permission android:name=\"android.permission.MOUNT_UNMOUNT_FILESYSTEMS\"/>",
        "<uses-permission android:name=\"android.permission.WRITE_SETTINGS\"/>"];

    return permission;
}

async function makewgt(request) {
    var phoneModules = request.platforms;
    var appid = request.manifast.id;

    // 项目下的manifast
    var appConfig = request.manifast;

    try {
        if (request.adsOpened) {
            var permissions = appConfig.plus.distribute.google.permissions;
            if (util.isEmptyObject(permissions)) {
                appConfig.plus.distribute.google.permissions = getadsPermissions();
            }
        }
        if (request.adid) {
            appConfig.plus.adid = request.adid;
        }
    } catch (e) { }

    if (containsIos(phoneModules)) {
        var appleProfile = appConfig.plus.distribute.apple;
        if(!appleProfile) {
            appleProfile = {};
        }
        if(appleProfile){
            appleProfile.appid = request.iosAppID;
            appleProfile.password = request.iosCertPassword;
            appleProfile.mobileprovision = request.iosProfile;
            appleProfile.p12 = request.iosCert;
            if (request.iphoneEnable && !request.ipadEnable) {
                appleProfile.devices = "iphone";
            } else if (!request.iphoneEnable && request.ipadEnable) {
                appleProfile.devices = "ipad";
            } else if (request.iphoneEnable && request.ipadEnable) {
                appleProfile.devices = "universal";
            }
            appConfig.plus.distribute.apple = appleProfile;
        }       
    }

    if (containsAndroid(phoneModules)) {
        var googleProfile = appConfig.plus.distribute.google;
        if(!googleProfile) {
            googleProfile = {};
        }
        if(googleProfile){
            googleProfile.packagename = request.androidPackageName;
            if (phoneModules.indexOf("ANDROID-GOOGLEPLAY") > -1) {
                googleProfile.aliasname = request.androidCertAlias;
                googleProfile.password = request.androidCertPassword;
                googleProfile.keystore = request.androidCert;
            } else if (phoneModules.indexOf("ANDROID-TEST") > -1) {
                googleProfile.aliasname = "";
                googleProfile.password = "";
                googleProfile.keystore = "html5plus://test";
            }
            appConfig.plus.distribute.google = googleProfile;
        }
    }
    var debugMode = request.debug;
    var needConfusion = request.needConfusion;
    var configJson = appConfig;

    var projectDir = request.project;
    if (projectutil.isUniAppVue(request)) {
        projectDir = projectutil.getCompilePath(request);
    }

    let tmp = os.tmpdir();
    tmp = tmp + "/.tmp/" + Date.now();
    tmp = util.pathReplaceSlash(tmp);
    if (fs.existsSync(tmp)) {
        fs.rmdirSync(tmp, {
            force: true,
            recursive: true
        });
    }
    fs.mkdirSync(tmp, {
        recursive: true
    })

    appConfig = manifestUtil.removePluginsByModule(appConfig);

    //编译后的manifest
    var manifestFile = projectDir + "/manifest.json";
    var appConfigJson = manifestUtil.setManifestDefaultValues(appConfig, manifestFile, request, "ALL", false, debugMode);
    var appidDir = tmp + "/" + appid;
    appidDir = await projectResources.copyPackAgeResources(request, needConfusion, true, projectDir, appidDir, appConfigJson);

    if (appConfig.hasOwnProperty("dependencies")) delete appConfig["dependencies"];
    if (appConfig.hasOwnProperty("unpackage")) delete appConfig["unpackage"];
    if (appConfig.hasOwnProperty("@wap2app_dev")) delete appConfig["@wap2app_dev"];

    var manifastDir = appidDir + "/.manifest";
    fs.mkdirSync(manifastDir, {
        recursive: true
    });
    var icons = appConfig.plus.distribute.icons;
    var splashScreen = appConfig.plus.distribute.splashscreen;
    var projectPath = request.project;
    if (containsIos(phoneModules)) {
        var appleProfile = appConfig.plus.distribute.apple;
        if (appleProfile) {
            var appleProfileJson = appConfigJson.plus.distribute.apple;
            var fileName = util.copyFile(projectPath, appleProfileJson.mobileprovision, "apple-mobileprovision",
                manifastDir);
            if (fileName) {
                appleProfileJson.mobileprovision = fileName;
            }
            var fileName = util.copyFile(projectPath, appleProfileJson.p12, "apple-private", manifastDir);
            if (fileName) {
                appleProfileJson.p12 = fileName;
            }
            appConfigJson.plus.distribute.apple = appleProfileJson;
        }
        if (icons) {
            var iosicon = icons.ios;
            if (iosicon) {
                var auto = iosicon.appstore;
                if (!util.isStringEmpty(auto)) {
                    var iOSIconJson = appConfigJson.plus.distribute.icons.ios;
                    var filename = util.copyFile(projectPath, auto, "appstore", manifastDir);
                    if (filename != null) {
                        appConfigJson.plus.distribute.icons.ios.appstore = filename;
                    }
                    var iPhoneIcon = iosicon.iphone;
                    if (iPhoneIcon != null) {
                        var iPhoneIconJson = appConfigJson.plus.distribute.icons.ios.iphone;
                        var ret = copyImageFiles(projectPath, manifastDir, iPhoneIcon, iPhoneIconJson, "icon-iphone-");
                        if (ret) {
                            for (var field in ret) {
                                iPhoneIconJson[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.icons.ios.iphone = iPhoneIconJson;
                        }
                    }
                    var iPadIcon = iosicon.ipad;
                    if (iPadIcon) {
                        var ipadIconJson = appConfigJson.plus.distribute.icons.ios.ipad;
                        var ret = copyImageFiles(projectPath, manifastDir, iPadIcon, ipadIconJson, "icon-ipad-");
                        if (ret) {
                            for (var field in ret) {
                                ipadIconJson[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.icons.ios.ipad = ipadIconJson;
                        }
                    }
                }
            }
        }

        if (splashScreen) {
            var iosSplashScreen = splashScreen.ios;
            var iosStyle = splashScreen.iosStyle;
            if (iosSplashScreen) {
                if ('default' == iosStyle || util.isStringEmpty(iosStyle)) {
                    var iPhoneSplashScreen = iosSplashScreen.iphone;
                    if (iPhoneSplashScreen) {
                        var iPhoneSplashScreenJson = appConfigJson.plus.distribute.splashscreen.ios.iphone;
                        //copyImageFiles(project, manifastDir, iPhoneSplashScreen, iPhoneSplashScreenJson, "splash-iphone-");
                        var ret = copyImageFiles(projectPath, manifastDir, iPhoneSplashScreen, iPhoneSplashScreenJson,
                            "splash-iphone-");
                        if (ret) {
                            for (var field in ret) {
                                iPhoneSplashScreenJson[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.splashscreen.ios.iphone = iPhoneSplashScreenJson;
                        }
                    }
                    var iPadSplashScreen = iosSplashScreen.ipad;
                    if (iPadSplashScreen) {
                        var ipadSplashScreenJson = appConfigJson.plus.distribute.splashscreen.ios.ipad;
                        //copyImageFiles(project, manifastDir, iPadSplashScreen, ipadSplashScreenJson, "splash-ipad-");
                        var ret = copyImageFiles(projectPath, manifastDir, iPadSplashScreen, ipadSplashScreenJson,
                            "splash-ipad-");
                        if (ret) {
                            for (var field in ret) {
                                iPadSplashScreen[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.splashscreen.ios.ipad = ipadSplashScreenJson;
                        }
                    }
                } else if ('storyboard' == iosStyle) {
                    var storyboard = iosSplashScreen.storyboard;
                    var storyboardObject = appConfigJson.plus.distribute.splashscreen.ios;
                    //var targetFileName = copyStoryBoard(project,manifastDir,storyboard,"storyboard-ios");
                    var fileName = util.copyFile(projectPath, storyboard, "storyboard-ios", manifastDir);
                    if (fileName) {
                        appConfigJson.plus.distribute.splashscreen.ios.storyboard = fileName;
                    }
                }
            }
        }
    }

    if (containsAndroid(phoneModules)) {
        var googleProfile = appConfig.plus.distribute.google;
        if (googleProfile) {
            var googleProfileJson = appConfigJson.plus.distribute.google;
            var filename = util.copyFile(projectPath, googleProfile.keystore, "google-keystore", manifastDir);
            if (filename) {
                appConfigJson.plus.distribute.google.keystore = filename;
            }
        }
        if (icons) {
            var androidIcon = icons.android;
            if (androidIcon) {
                var androidIconJson = appConfigJson.plus.distribute.icons.android;
                var ret = copyImageFiles(projectPath, manifastDir, androidIcon, androidIconJson, "icon-android-");
                if (ret) {
                    for (var field in ret) {
                        androidIconJson[field] = ret[field];
                    }
                    appConfigJson.plus.distribute.icons.android = androidIconJson;
                }
            }
        }
        if (splashScreen) {
            var androidStyle = splashScreen.androidStyle;
            if ("default" == androidStyle || util.isStringEmpty(androidStyle)) {
                var androidSplashScreen = splashScreen.android;
                if (androidSplashScreen) {
                    var androidSplashScreenJson = appConfigJson.plus.distribute.splashscreen.android;
                    var ret = copyImageFiles(projectPath, manifastDir, androidSplashScreen, androidSplashScreenJson, "splash-android-");
                    if (ret) {
                        for (var field in ret) {
                            androidSplashScreenJson[field] = ret[field];
                        }
                        appConfigJson.plus.distribute.splashscreen.android = androidSplashScreenJson;
                    }
                }
            }
        }
    }

    var checkiosPlatform = containsIos(phoneModules);
    var plugins = appConfig.plus.distribute.plugins;
    if (plugins) {
        var push = plugins.push;
        if (push) {
            var gexinPush = push.igexin;
            if (gexinPush) {
                var gexinICon = gexinPush.icons;
                if (gexinICon) {
                    var gexinPushICon = gexinICon.push;
                    if (gexinPushICon) {
                        var gexinPushIConObject = appConfigJson.plus.distribute.plugins.push.igexin.icons.push;
                        // copyImageFiles(project, manifastDir, gexinPushICon, gexinPushIConObject, "icon-igexin-push-");
                        var ret = copyImageFiles(projectPath, manifastDir, gexinPushICon, gexinPushIConObject, "icon-igexin-push-");
                        if (ret) {
                            for (var field in ret) {
                                gexinPushIConObject[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.plugins.push.igexin.icons.push = gexinPushIConObject;
                        }
                    }
                    var gexinSmallICon = gexinICon.small;
                    if (gexinSmallICon) {
                        var gexinSmallIConObject = appConfigJson.plus.distribute.plugins.push.igexin.icons.small;
                        var ret = copyImageFiles(projectPath, manifastDir, gexinSmallICon, gexinSmallIConObject, "icon-igexin-small-");
                        if (ret) {
                            for (var field in ret) {
                                gexinSmallIConObject[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.plugins.push.igexin.icons.small = gexinSmallIConObject;
                        }
                    }
                }
            }

            var uniPush = push.unipush;
            if (uniPush) {
                var gexinICon = uniPush.icons;
                if (gexinICon) {
                    var gexinPushICon = gexinICon.push;
                    if (gexinPushICon) {
                        var gexinPushIConObject = appConfigJson.plus.distribute.plugins.push.unipush.icons.push;
                        // copyImageFiles(project, manifastDir, gexinPushICon, gexinPushIConObject, "icon-igexin-push-");
                        var ret = copyImageFiles(projectPath, manifastDir, gexinPushICon, gexinPushIConObject, "icon-unipush-push-");
                        if (ret) {
                            for (var field in ret) {
                                gexinPushIConObject[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.plugins.push.unipush.icons.push = gexinPushIConObject;
                        }
                    }
                    var gexinSmallICon = gexinICon.small;
                    if (gexinSmallICon) {
                        var gexinSmallIConObject = appConfigJson.plus.distribute.plugins.push.unipush.icons.small;
                        var ret = copyImageFiles(projectPath, manifastDir, gexinSmallICon, gexinSmallIConObject, "icon-igexin-small-");
                        if (ret) {
                            for (var field in ret) {
                                gexinSmallIConObject[field] = ret[field];
                            }
                            appConfigJson.plus.distribute.plugins.push.unipush.icons.small = gexinSmallIConObject;
                        }
                    }
                }
            }

            var fcmPush = push.fcm;
            if (fcmPush) {
                var config_ios = fcmPush.config_ios;
                if (!util.isStringEmpty(config_ios) && checkiosPlatform) {
                    if (fs.existsSync(config_ios)) {
                        util.copyFile(projectPath, config_ios, "GoogleService-Info", manifastDir);
                    }
                    else {
                        //throw "iOS平台Google Analytics的配置文件["+config_ios+"]不存在";
                    }
                }
                var config_android = fcmPush.config_android;
                if (!util.isStringEmpty(config_android) && containsAndroid(phoneModules)) {
                    if (fs.existsSync(config_android)) {
                        util.copyFile(projectPath, config_android, "google-services", manifastDir);
                    } else {
                        //throw "Android平台Google Analytics的配置文件["+config_ios+"]不存在;
                    }
                }
            }
        }

        var statics = plugins.statics;
        if (statics) {
            var googleStatics = statics.google;
            if (googleStatics != null) {
                var config_ios = googleStatics.config_ios;
                if (!util.isStringEmpty(config_ios) && checkiosPlatform) {
                    if (fs.existsSync(config_ios)) {
                        util.copyFile(projectPath, config_ios, "GoogleService-Info", manifastDir);
                    }
                    else {
                        throw "iOS平台Google Analytics的配置文件[" + config_ios + "]不存在";
                    }
                }
                var config_android = googleStatics.config_android;
                if (!util.isStringEmpty(config_android) && containsAndroid(phoneModules)) {
                    if (fs.existsSync(config_android)) {
                        util.copyFile(projectPath, config_android, "google-services", manifastDir);
                    } else {
                        throw "Android平台Google Analytics的配置文件[" + config_android + "]不存在";
                    }
                }
            }
        }
    }

    if (projectutil.isUniAppVue(request)) {
        var plusObject = appConfigJson.plus;
        if (plusObject) {
            var nativePluginsObject = plusObject.nativePlugins;
            var pluginDirList = [];
            var notcopyFileList = {};

            if (nativePluginsObject) {
                for (var pluginId in nativePluginsObject) {
                    var pluginIdObject = nativePluginsObject[pluginId];
                    if (!pluginIdObject) {
                        throw "插件[" + pluginId + "]不合法：manifest.json配置错误，不符合插件配置规范";
                    }

                    var isCloud = false;
                    var pluginName = pluginId;
                    if (pluginIdObject.hasOwnProperty("__plugin_info__")) {
                        var pluginInfoObject = pluginIdObject["__plugin_info__"];
                        if (pluginInfoObject) {
                            pluginName = pluginInfoObject.name;
                            isCloud = pluginInfoObject.isCloud;
                            delete appConfigJson.plus.nativePlugins[pluginId]["__plugin_info__"];
                        }
                    }
                    
                    var projectNativePluginDir = projectPath + "/nativeplugins/" + pluginId;
                    var packageFile = projectPath + "/nativeplugins/" + pluginId + "/package.json";

                    if (isCloud) {
                        if (!fs.existsSync(projectNativePluginDir) || !fs.statSync(projectNativePluginDir).isDirectory() || fs.existsSync(packageFile)) {
                            continue;
                        }
						if(fs.statSync(projectNativePluginDir).isDirectory()){
							let fileArr = fs.readdirSync(projectNativePluginDir);
							if (fileArr.length < 1) {
							    continue;
							}
						}
                    } else {
                        if (pluginId.toLowerCase() == "bxm-ad") {
                            continue;
                        }
                        if (!fs.existsSync(projectNativePluginDir) || !fs.statSync(projectNativePluginDir).isDirectory()) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：该插件在nativePlugins目录下不存在";
                        }
                        if (!fs.existsSync(packageFile)) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json不存在";
                        }

                        var iOSDir = projectNativePluginDir + "/ios";
                        var androidDir = projectNativePluginDir + "/android";
                        if (!fs.existsSync(iOSDir) && !fs.existsSync(androidDir)) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：ios或android目录不存在";
                        }
                        notcopyFileList[iOSDir] = {};
                        notcopyFileList[androidDir] = {};

                        if (fs.statSync(packageFile).size <= 0) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json内容为空";
                        }

                        var readFile = fs.readFileSync(packageFile, 'utf8');
                        var json = hjson.parse(readFile);
                        if (util.isEmptyObject(json)) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json内容不是合法的json";
                        }
                        var dpNativePluginObject = json["_dp_nativeplugin"];
                        if (util.isEmptyObject(dpNativePluginObject)) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中不存在_dp_nativeplugin节点";
                        }

                        var iosObject = dpNativePluginObject.ios;
                        var androidObject = dpNativePluginObject.android;
                        if (containsIos(phoneModules) && !util.isEmptyObject(iosObject) && !fs.existsSync(iOSDir)) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：ios目录不存在";
                        }
                        if (containsAndroid(phoneModules) && !util.isEmptyObject(androidObject) && !fs.existsSync(androidDir)) {
                            throw "插件[" + pluginName + "(" + pluginId + ")]不合法：android目录不存在";
                        }
                        if (containsIos(phoneModules) && !util.isEmptyObject(iosObject)) {
                            var pluginsArray = iosObject.plugins;
                            if (!pluginsArray || pluginsArray.length < 1) {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.plugins节点下存在为空的对象";
                            }
                            for (var pluginElement in pluginsArray) {
                                var pluginObject = pluginsArray[pluginElement];
                                if (!pluginObject) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.plugins节点下存在为空的对象";
                                }
                                var pluginTypeElement = pluginObject.type;
                                if (!pluginTypeElement) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.plugins节点下存在type为空的对象";
                                }
                                var pluginType = pluginTypeElement;
                                if (util.isStringEmpty(pluginType) || (pluginType != "module" && pluginType != "component")) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.plugins节点下存在type值不合法的对象，type必须为module或component";
                                }
                                var pluginNameElement = pluginObject.name;
                                if (!pluginNameElement || util.isStringEmpty(pluginNameElement)) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.plugins节点下存在name为空的对象";
                                }
                                var pluginClassElement = pluginObject.class;
                                if (!pluginClassElement || util.isStringEmpty(pluginClassElement)) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.plugins节点下存在class为空的对象";
                                }
                            }
                            var integrateTypeElement = iosObject.integrateType;
                            if (!integrateTypeElement) {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.integrateType节点不存在或内容为空";
                            }
                            if (util.isStringEmpty(integrateTypeElement)) {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.integrateType节点内容为空";
                            }

                            if (integrateTypeElement != "framework" && integrateTypeElement != "library") {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.ios.integrateType节点值不合法，integrateType必须为framework或library";
                            }
                            delete notcopyFileList[iOSDir];
                        }
                        if (containsAndroid(phoneModules) && !util.isEmptyObject(androidObject)) {

                            var pluginsArray = androidObject.plugins;
                            if (!pluginsArray || pluginsArray.length < 1) {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.plugins节点下存在为空的对象";
                            }
                            for (var pluginElement in pluginsArray) {
                                var pluginObject = pluginsArray[pluginElement];
                                if (!pluginObject) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.plugins节点下存在为空的对象";
                                }
                                var pluginTypeElement = pluginObject.type;
                                if (!pluginTypeElement) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.plugins节点下存在type为空的对象";

                                }
                                var pluginType = pluginTypeElement;
                                if (util.isStringEmpty(pluginType) || (pluginType != "module" && pluginType != "component")) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.plugins节点下存在type值不合法的对象，type必须为module或component";
                                }
                                var pluginNameElement = pluginObject.name;
                                if (!pluginNameElement || util.isStringEmpty(pluginNameElement)) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.plugins节点下存在name为空的对象";
                                }
                                var pluginClassElement = pluginObject.class;
                                if (!pluginClassElement || util.isStringEmpty(pluginClassElement)) {
                                    throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.plugins节点下存在class为空的对象";
                                }
                            }
                            var integrateTypeElement = androidObject.integrateType;
                            if (!integrateTypeElement) {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.integrateType节点不存在或内容为空";
                            }
                            if (util.isStringEmpty(integrateTypeElement)) {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.integrateType节点内容为空";
                            }

                            if (integrateTypeElement != "aar" && integrateTypeElement != "jar") {
                                throw "插件[" + pluginName + "(" + pluginId + ")]不合法：package.json中_dp_nativeplugin.android.integrateType节点值不合法，integrateType必须为aar或jar";
                            }
                            delete notcopyFileList[androidDir];
                        }
                        var pd = appidDir + "/nativeplugins/" + pluginId;
                        if (!fs.existsSync(pd)) {
                            fs.mkdirSync(pd, { recursive: true })
                        }
                        let jsonData = JSON.stringify(json);
                        fs.writeFileSync(pd + "/package.json", jsonData);
                    }
                    pluginDirList.push(projectNativePluginDir);
                }

                if (pluginDirList.length > 0) {
                    var appidNativePluginsDir = appidDir + "/nativeplugins";
                    if (!fs.existsSync(appidNativePluginsDir)) {
                        fs.mkdirSync(appidNativePluginsDir, { recursive: true })
                    }
                    for (var pluginDir in pluginDirList) {
                        var pluginPath = pluginDirList[pluginDir];
                        var pluginName = path.basename(pluginPath);
                        var appidPlguin = appidNativePluginsDir + "/" + pluginName;
                        if (!fs.existsSync(appidPlguin)) {
                            fs.mkdirSync(appidPlguin);
                        }
                        util.copyDirectory(pluginPath, appidPlguin, (filePath) => {
                            if (fs.statSync(filePath).isDirectory()) {
                                if (notcopyFileList.hasOwnProperty(filePath)) return false;
                            }
                            return true;
                        });
                    }
                }
            }
        }
    }

    copyNativeResources(request,appidDir);
    
    let data = JSON.stringify(appConfigJson);
    fs.writeFileSync(appidDir + "/manifest.json", data);
    var wgtfile  =  path.join(os.tmpdir(),"/" + appid + ".wgt");
    
    await util.zipRecource(tmp,wgtfile);
    
    var limitSize = 40;
    if (fs.existsSync(wgtfile)) {
        // if (request.wgtLimitSize > limitSize) {
        //     limitSize = request.wgtLimitSize;
        // }
        // var fsInfo = fs.statSync(wgtfile);
        // if (fsInfo.size / 1024 > limitSize * 1024) {
        //     var limitMsg = "App包大小不可超过40M，请清除项目下无用文件再重试；如需超限打包，请联系bd@dcloud.io";
        //     if (!util.isStringEmpty(request.wgtLimitMsg)) {
        //         limitMsg = request.wgtLimitMsg;
        //     }
        //     throw limitMsg;
        // }
        return wgtfile;
    }
    return;
}

/*
项目下的AndroidManifest.xml、iOSInfo.plist文件，以及nativeResources：
1. Android平台，需要提交云端的文件及目录
AndroidManifest.xml、nativeResources/android
2. iOS平台，需要提交云端的文件及目录
iOSInfo.plist、nativeResources/ios
*/
async function copyNativeResources(request,appidDir){
    if(!request || !request.project) return;
    if(!appidDir) return;
    var platforms = request.platforms;
    var mustCopyfile = [];
    var mustCopyDir = [];
    
    if(containsAndroid(platforms)){
        var androidManifest = path.join(request.project,"AndroidManifest.xml");
        if(fs.existsSync(androidManifest)){
            mustCopyfile.push("AndroidManifest.xml");
        }
        var androidDir = path.join(request.project,"nativeResources","android");
        if(fs.existsSync(androidDir)){
            mustCopyDir.push(path.join("nativeResources","android"));
        }
    }
    
    if(containsIos(platforms)){
        var iOSInfo = path.join(request.project,"iOSInfo.plist");
        if(fs.existsSync(iOSInfo)){
            mustCopyfile.push("iOSInfo.plist");
        }
        var iosDir = path.join(request.project,"nativeResources","ios");
        if(fs.existsSync(iosDir)){
            mustCopyDir.push(path.join("nativeResources","ios"));
        }
    }
    
    for (var index in mustCopyfile) {
        var fileName = mustCopyfile[index];
        var filePath = path.join(request.project,fileName);
        if(fs.existsSync(filePath)){
            fs.copyFileSync(filePath, path.join(appidDir,fileName));
        }
    }
    
    for (var dirIndex in mustCopyDir) {
        var dir = mustCopyDir[dirIndex];
        var dirPath  = path.join(request.project,dir);
        
        if(fs.existsSync(dirPath)){
            var destDir = path.join(appidDir,dir);
            if (!fs.existsSync(destDir)) {
                fs.mkdirSync(destDir,{recursive:true})
            }
            util.copyDirectory(dirPath,destDir);
        }
    }
    
}

async function makeapp(params) {
    var ret;
    try {
        var res = await makewgt(params)
        ret = { result: res }
    } catch (e) {
        if(typeof e === 'string'){
           ret = { error: e } 
        }
    }
    return ret;
}

// 导出
module.exports = {
    makeapp: makeapp
}
