var fs = require("fs")
var os = require("os")
var util = require("./util")
var hjson = require("./hjson.min")
var projectUtil = require("./projectutil")
var pathUtil = require("path")

var pluginsModulesConfig = {
	OAuth: "oauth",
	Maps: "maps",
	Geolocation: "geolocation",
	Payment: "payment",
	Push: "push",
	Share: "share",
	Speech: "speech",
	Statistic: "statics"
};

function removePluginsByModule(json) {
	var appConfig = json;
	var plugins = appConfig["plus"]["distribute"]["plugins"];
	var modules = appConfig.permissions;
	if (!plugins) {
		return appConfig;
	}
	if (modules) {
		for (var item in pluginsModulesConfig) {
			var name = pluginsModulesConfig[item];
			if (!modules || !modules[item]) {
				delete appConfig["plus"]["distribute"]["plugins"][name];
			}
		}
	}
	return appConfig;
}

function addDefaultObjectToParent(parent, name, obj) {
	if (!parent.name) {
		return obj;
	}
	return parent[name];
}

function fileExistsByproject(projectPath, path) {
	if (!path) return;
	if (fs.existsSync(path)) {
		return path;
	} else if (fs.existsSync(pathUtil.join(projectPath, path))) {
		return pathUtil.join(projectPath, path);
	}
	return;
}

function mergeTabBar(webviewParameterObject, launchwebviewObject) {
	try {
		if (!webviewParameterObject) return false;
		var tabBarKey = "tabBar";
		var tabBarObject = webviewParameterObject[tabBarKey];
		if (!tabBarObject) return false;
		var heightStr = tabBarObject.height;
		if (util.isStringEmpty(heightStr)) return;
		heightStr = heightStr.replace("px", "");
		var height = Number(heightStr);
		if (height <= 0) return false;
		if (webviewParameterObject) {
			var webviewParameterTopString = webviewParameterObject.top;
			var webviewParameterBottomStrng = webviewParameterObject.bottom;
			if (!util.isStringEmpty(webviewParameterObject.top) && !util.isStringEmpty(webviewParameterObject.bottom)) {
				return false;
			}
		}
		var top = 0;
		var bottom = 0;
		var isSet = false;
		var tabBarPosition = tabBarObject.position;
		if (!util.isStringEmpty(tabBarPosition) && tabBarPosition.equals("top")) {
			isSet = true;
			top = height;
		}

		if (!isSet) {
			bottom = height;
		}
		launchwebviewObject.top = top.toString() + "px";
		launchwebviewObject.bottom = bottom.toString() + "px";
		return true;
	} catch (e) {

	}
	return false;
}

function mergeStatusbar(webviewParameterObject, launchwebviewObject) {
	var statusbarKey = "statusbar";
	var statusbarElement = webviewParameterObject[statusbarKey];
	if (statusbarElement) {
		delete launchwebviewObject[statusbarKey];
		if (typeof(statusbarElement) == 'object') {
			launchwebviewObject.statusbarKey = statusbarElement;
		}
		return true;
	}
	return false;
}

function mergeTitleNviewObject(pagesElement, globalElement) {
	try {
		if (typeof(pagesElement) == 'object') {
			var titleNViewObject = pagesElement;
			var globalTitleNViewObject = globalElement;
			for (var key in globalElement) {
				var entryElement = globalElement[key];
				if (!titleNViewObject.hasOwnProperty(key)) {
					titleNViewObject[key] = entryElement;
					continue;
				}
				var keyElement = titleNViewObject[key];
				if (typeof(keyElement) == 'object') {
					mergeTitleNviewObject(keyElement, entryElement);
					continue;
				}
			}
		}
	} catch (e) {
		console.log(e);
	}
}

function mergeLaunchWebView(project, plusJson, appid) {

	var sitemapFile = project.project + "/sitemap.json";
	if (!fs.existsSync(sitemapFile)) {
		return;
	}
	var readFile = fs.readFileSync(sitemapFile, 'utf8');
	var sitemapJson = hjson.parse(readFile);
	if (util.isEmptyObject(sitemapJson)) return;
	var globalObject = sitemapJson.global;
	var globalWebviewParameterObject;
	var webviewParameterKey = "webviewParameter";
	if (globalObject) {
		globalWebviewParameterObject = globalObject[webviewParameterKey];
	}

	if (globalWebviewParameterObject) {
		// mergeStyle
		var statusbarKey = "statusbar";
		var statusbarObject = globalWebviewParameterObject[statusbarKey];
		if (statusbarObject) {
			var styleKey = "style";
			var styleElement = statusbarObject[styleKey];
			if (styleElement) {
				var plusStatusbarElement = plusJson[statusbarKey];
				if (!plusStatusbarElement) {
					plusJson[statusbarKey][styleKey] = {};
				}
				plusJson[statusbarKey][styleKey] = styleElement;
			}
		}
	}

	var pagesArray = sitemapJson.pages;
	if (!pagesArray) return;

	var pageObject;
	for (var index in pagesArray) {
		var currentPageObject = pagesArray[index];
		if (!currentPageObject) continue;
		var webviewId = currentPageObject.webviewId;
		if (!util.isStringEmpty(webviewId) && webviewId == appid) {
			pageObject = currentPageObject;
			break;
		}
	}
	if (!pageObject) return;
	var launchwebviewKey = "launchwebview";
	if (!plusJson[launchwebviewKey]) {
		plusJson[launchwebviewKey] = {};
	}
	var launchwebviewObject = plusJson[launchwebviewKey];
	var webviewParameterObject = pageObject[webviewParameterKey];
	if (!webviewParameterObject) return;

	var added = false;
	if (mergeTabBar(webviewParameterObject, launchwebviewObject) == true) {
		plusJson[launchwebviewKey] = launchwebviewObject;
		var secondwebviewKey = "secondwebview";
		if (!plusJson.hasOwnProperty(secondwebviewKey)) {
			plusJson[secondwebviewKey] = {};
		}
		plusJson[secondwebviewKey]["mode"] = "parent";
		added = true;
	}
	if (mergeStatusbar(webviewParameterObject, launchwebviewObject) == true) {
		if (!added) {
			plusJson[launchwebviewKey] = launchwebviewObject;
			added = true;
		}
	}

	var titleNviewKey = "titleNView";
	var titleNViewElement = webviewParameterObject[titleNviewKey];
	var typeTiel = typeof(titleNViewElement);
	if (!titleNViewElement && typeof(titleNViewElement) != 'boolean') {
		return;
	}
	if (!added) {
		plusJson[launchwebviewKey] = launchwebviewObject;
	}

	if (!globalObject) {
		delete launchwebviewObject[titleNviewKey];
		if (typeof(titleNViewElement) == 'object') {
			launchwebviewObject[titleNviewKey] = titleNViewElement;
		}
		return;
	}

	var globalTitleNViewElement = globalWebviewParameterObject[titleNviewKey];
	if (!globalTitleNViewElement) {
		delete launchwebviewObject[titleNviewKey];
		if (typeof(titleNViewElement) == 'object') {
			launchwebviewObject[titleNviewKey] = titleNViewElement;
		}
		return;
	}

	mergeTitleNviewObject(titleNViewElement, globalWebviewParameterObject);
	delete launchwebviewObject[titleNviewKey];
	if (typeof(titleNViewElement) == 'object') {
		launchwebviewObject[titleNviewKey] = titleNViewElement;
	}
}

function setManifestDefaultValues(config, manifestFile, project, platform, addDebug, debugMode) {
	var readFile = fs.readFileSync(manifestFile, 'utf8');
	var json = hjson.parse(readFile);
	var modules = json.permissions;
	var projectPath = project.project;
	if (!modules) {
		config["permissions"] = {};
	}

	if (config.icons) {
		var path48 = json.icons["48"];
		var filePath = fileExistsByproject(projectPath, path48);
		if (filePath) {
			json.icons["48"] = filePath;
		}
	}
    
	var plus = config.plus;
	var plusJson = addDefaultObjectToParent(json, "plus", plus);
    if(!plusJson) return;
    
	if (plus.adid && plus.adid != "") {
		json.plus.adid = plus.adid;
	}

	try {
		if (projectUtil.isWap2AppProject(project)) {
			var secondwebview = plus.secondwebview;
			if (!secondwebview) {
				secondwebview = {};
			}
			if (util.isStringEmpty(secondwebview.mode)) {
				secondwebview.mode = "behind";
			}
			secondwebview.launch_path = "_www/client_index.html";
			secondwebview.id = "__W2A_CONTEXT_";
			json.plus.secondwebview = secondwebview;
			mergeLaunchWebView(project, json.plus, config.id);
		}
	} catch (e) {
		console.error(e);
	}

	if (util.isStringEmpty(plus.ramcachemode)) {
		json.plus.ramcachemode = plus.ramcachemode;
	}

	if (plus.splashscreen) {
		var splashscreenJson = addDefaultObjectToParent(plusJson, "splashscreen", plus.splashscreen);
		splashscreenJson.autoclose = plus.splashscreen.autoclose;
		json.plus.splashscreen = splashscreenJson;
	}
    
	var distributeJson = addDefaultObjectToParent(plusJson, "distribute", plusJson.distribute);
    if(!distributeJson) return;
    
	if (plus.distribute.debug == true || addDebug == true) {
		json.plus.distribute.debug = true;
	}

	if (debugMode) {
		json.plus.distribute["debug"] = true;
		json.plus.distribute["syncDebug"] = true;
		var permissionsObj = json["permissions"];
		if (!permissionsObj) {
			permissionsObj["Console"] = {
				description: "调试输出日志"
			};
			json["permissions"] = permissionsObj;
		}
	}

	if (!distributeJson.hasOwnProperty("orientation")) {
		json.plus.distribute.orientation = "portrait-primary";
	}

	var icons = plus.distribute.icons;
	var iconsJson = addDefaultObjectToParent(distributeJson, "icons", icons);
	var splashScreen = plus.distribute.splashscreen;

	var isIPhone = false;
	var isIPad = false;

	if (platform == "ALL" || platform == "IOS") {
        var appleProfile = plus.distribute.apple;
        if(appleProfile){
                var appleProfileJson = addDefaultObjectToParent(distributeJson, "apple", appleProfile);
                appleProfileJson["appid"] = appleProfile.appid;
                appleProfileJson["password"] = appleProfile.password;
                appleProfileJson["mobileprovision"] = appleProfile.mobileprovision;
                appleProfileJson["p12"] = appleProfile.p12;
                if (debugMode) {
                    var listArr = ["Add :UIFileSharingEnabled bool true"];
                    appleProfileJson["plistcmds"] = listArr;
                }
                
                if (!appleProfile.devices) {
                    appleProfile.devices = "universal";
                }
                var devices = appleProfile.devices;
                if (devices == "universal") {
                    isIPhone = true;
                    isIPad = true;
                }
                if (devices == "ipad") {
                    isIPad = true;
                }

                if (devices == "iphone") {
                    isIPhone = true;
                }
                appleProfileJson["devices"] = devices;
                json.plus.distribute.apple = appleProfileJson;
        }
        if (icons) {
            var iOSIcon = icons.ios;
            if(iOSIcon){
                var iOSIconJson = addDefaultObjectToParent(iconsJson, "ios", iOSIcon);
                if (typeof(iOSIcon.prerendered) != 'boolean') {
                    iOSIconJson["prerendered"] = "false";
                }

                iOSIconJson["appstore"] = iOSIcon.appstore;
                if (iOSIconJson.hasOwnProperty("auto")) {
                    delete iOSIconJson["auto"];
                }
                if (isIPhone) {
                    iOSIconJson["iphone"] = iOSIcon.iphone;
                }
                if (isIPad) {
                    iOSIconJson["ipad"] = iOSIcon.ipad;
                }
                json.plus.distribute.icons.ios = iOSIconJson
            }
        }
	}

	if (platform == "ALL" || platform == "ANDROID") {
		var googleProfile = plus.distribute.google;
		if (googleProfile) {
			var googleProfileJson = addDefaultObjectToParent(distributeJson, "google", googleProfile);
			googleProfileJson["packagename"] = googleProfile.packagename;
			googleProfileJson["password"] = googleProfile.password;
			googleProfileJson["aliasname"] = googleProfile.aliasname;
			googleProfileJson["keystore"] = googleProfile.keystore;
			googleProfileJson["custompermissions"] = true;
			json.plus.distribute.google = googleProfileJson;
		} else {
			var googleProfileJson = addDefaultObjectToParent(distributeJson, "google", {});
			googleProfileJson["custompermissions"] = true;
			json.plus.distribute.google = googleProfileJson;
		}

		if (icons) {
			var androidIcon = icons.android;
			var androidIconJson = addDefaultObjectToParent(iconsJson, "android", androidIcon);
            if(androidIconJson){
                var dpiConifg = ["hdpi", "xhdpi", "xxhdpi", "xxxhdpi"];
                for (var dpiItem in dpiConifg) {
                    var dpivalue = dpiConifg[dpiItem];
                    var itemPath = androidIconJson[dpivalue];
                    var existsfile = false;
                    delete androidIconJson[dpivalue];
                    if (itemPath) {
                        var filePath = fileExistsByproject(projectPath, itemPath);
                        if (filePath) {
                            androidIconJson[dpivalue] = filePath;
                        }
                    }
                }
                json.plus.distribute.icons.android = androidIconJson;
            }
		}
		if (splashScreen) {
			var splashScreenJson = addDefaultObjectToParent(distributeJson, "splashscreen", splashScreen);
			var androidSplashScreen = splashScreen.android;
			var androidSplashScreenJson = splashScreenJson.android
			if (!androidSplashScreen && !androidSplashScreenJson) {
				if (json.plus.distribute.android && json.plus.distribute.android.mdpi) {
					delete json.plus.distribute.android.mdpi;
				}
				if (json.plus.distribute.android && json.plus.distribute.android.ldpi) {
					delete json.plus.distribute.android.ldpi;
				}
			}
		}
	}

	var plugins = plus.distribute.plugins;
	if (plugins) {
		var pluginsJson = addDefaultObjectToParent(distributeJson, "plugins", plugins);
		var statics = plugins.statics;
		if (statics) {
			var staticsJson = addDefaultObjectToParent(pluginsJson, "statics", statics);
			if (util.isStringEmpty(statics.description)) {
				staticsJson.description = "管理消息推送能力"
			}
			pluginsJson.statics = staticsJson;
			if (!statics.umeng && !statics.google) {
				delete pluginsJson.statics
				delete plugins.statics
			}
			var speech = plugins.speech;
			var baiduEnable = true;
			var speechBaidu;
			if(speech) { speechBaidu = speech.baidu };
			if (!speechBaidu ||
				util.isStringEmpty(speechBaidu.apikey) ||
				util.isStringEmpty(speechBaidu.appid) ||
				util.isStringEmpty(speechBaidu.secretkey)) {
				baiduEnable = false;
			}

			if (speech && !speech.ifly && !baiduEnable) {
				delete pluginsJson.speech;
				delete plugins.speech
			}
		}

		var share = plugins.share;
		if (share) {
			var sina = share.sina;
			var tencent = share.tencent;
			var weixin = share.weixin;
			var qq = share.qq;
			if (!sina && !tencent && !weixin && !qq) {
				delete pluginsJson.share;
				delete plugins.share
			} else {
				var shareJson = addDefaultObjectToParent(pluginsJson, "share", share);
				if (sina) {
                    //  js不需要 util.isStringEmpty(sina.appsecret)
					if (util.isStringEmpty(sina.appkey) || util.isStringEmpty(sina.redirect_uri)) {
						delete shareJson.sina;
					}
				}
				if (tencent) {
					if (util.isStringEmpty(tencent.appkey) || util.isStringEmpty(tencent.appsecret) || util
						.isStringEmpty(tencent.redirect_uri)) {
						delete shareJson.tencent;
					}
				}
				if (weixin) {
					if (util.isStringEmpty(weixin.appid)) {
						delete shareJson.weixin;
					}
				}
				if (qq) {
					if (util.isStringEmpty(qq.appid)) {
						delete shareJson.qq;
					}
				}

				pluginsJson.share = shareJson;
			}
		}

		var push = plugins.push;
		if (push) {
			var pushJson = addDefaultObjectToParent(pluginsJson, "push", push);
			if (typeof(push.cover) == 'boolean') {
				pushJson.cover = false;
			}
			if (util.isStringEmpty(push.description)) {
				pushJson.description = "管理消息推送能力";
			}

			var iGexinPush = push.igexin;
			var uniPush = push.unipush;
			var miPush = push.mipush;
			var fcmPush = push.fcm;
			if (!iGexinPush && !miPush && !uniPush && !fcmPush) {
				delete pushJson.push;
				delete plugins.push
			}

			pluginsJson.push = pushJson;
		}

		var map = plugins.maps

		if (map != null) {
			var mapJson = addDefaultObjectToParent(pluginsJson, "maps", map);

			if (util.isStringEmpty(map.description)) {
				mapJson.description = "地图插件";
			}

			var baidumap = map.baidu;
			var amap = map.amap;
			var googlemap = map.google;
			if (!baidumap && !amap && !googlemap) {
				delete mapJson.maps;
				delete plugins.maps
			}
			pluginsJson.maps = mapJson;
		};

		if (pluginsJson) {
			for (var item in pluginsModulesConfig) {
				var name = pluginsModulesConfig[item];
				if (!plugins[name]) {
					delete pluginsJson[name]
				}
			}
		}
		json.plus.distribute.plugins = pluginsJson;
	}

	return json;
}

function removeConfigNodes(appconfig, needConfusion) {
	if (!needConfusion && appconfig.hasOwnProperty("plus")) {
		delete appconfig.plus.confusion;
	}
	if (appconfig.hasOwnProperty("dependencies")) delete appconfig.plus.dependencies;
	if (appconfig.hasOwnProperty("unpackage")) delete appconfig.plus.unpackage;
	if (appconfig.hasOwnProperty("@wap2app_dev")) delete appconfig.plus["@wap2app_dev"];
	if (appconfig.hasOwnProperty("plus")) {
		delete appconfig.plus.distribute;
	}
	return appconfig;
}

module.exports = {
	removePluginsByModule: removePluginsByModule,
	setManifestDefaultValues: setManifestDefaultValues,
	removeConfigNodes: removeConfigNodes
}
