var projectutil = require('./projectutil')
var util = require("./util")
var fs = require("fs")
var path = require("path")
var child_process = require("child_process");
var hx = require("hbuilderx");

function enable(project) {
    return projectutil.isWap2AppProject(project);
}

function ProcessorResult(srcDir, destDir, compileSrcFiles, compileDestFiles, sourceMapFiles, deleteDestFile) {
    return {
        srcDir: srcDir,
        destDir: destDir,
        compileSrcFiles: compileSrcFiles,
        compileDestFiles: compileDestFiles,
        sourceMapFiles: sourceMapFiles,
        deleteDestFile: deleteDestFile
    }
}

const serveIndexcss = "server_index_append.css";

function processFile(project, projectDir, destDir, file, isProduction) {
    if (util.isStringEmpty(project.nodePath)) return;
    if (!fs.existsSync(project.nodePath)) return;
    if (!enable(project)) return;
    var wap2appAppidAppendCSSFile = path.join(projectDir, project.id + ".append.css");
    var compileSrcFiles = [];
    var compileDestFiles = [];
    if (wap2appAppidAppendCSSFile == file) {
        if (fs.existsSync(wap2appAppidAppendCSSFile)) {
            var tofile = path.join(destDir, serveIndexcss);
            fs.copyFileSync(wap2appAppidAppendCSSFile, tofile);
            compileSrcFiles.push(path.extname(wap2appAppidAppendCSSFile));
            compileDestFiles.push(path.extname(tofile));
            return ProcessorResult(projectDir, destDir, compileDestFiles, compileDestFiles);
        } else {
            return ProcessorResult(projectDir, destDir, compileSrcFiles, compileDestFiles, serveIndexcss);
        }
    }

    var form = util.pathReplaceSlash(projectDir);
    var filePath = "";
    if (fs.existsSync(file)) {
        filePath = file;
    }
    var appRoot = hx.env.appRoot;
    var toolsDir = path.join(appRoot, "plugins", "launcher", "tools", "nview");
    var to = util.pathReplaceSlash(destDir);
    var indexjs = path.join(toolsDir, "index.js");
    var cmdArray = [];

    cmdArray.push(indexjs)
    cmdArray.push(form);
    cmdArray.push(filePath);
    cmdArray.push(to);
    if (isProduction == true) {
        cmdArray.push("production");
    } else {
        cmdArray.push("development");
    }

    process.chdir(toolsDir);

    var result = child_process.spawnSync(project.nodePath, cmdArray);
    var outPut = result.output.toString();
    if (outPut) {
        try {
            var replaceStr = outPut.replace(/^(\s|,)+|(\s|,)+$/g, '');
            var json = JSON.parse(replaceStr);
            var compileSrcFiles = [];
            var compileDestFiles = [];
            var sourceMapFiles = [];
            if (json) {
                var exclude = json.exclude;
                if (exclude) {
                    for (var oneexclude of exclude) {
                        if (!util.isStringEmpty(oneexclude)) {
                            compileSrcFiles.push(oneexclude);
                        }
                    }
                }
                var compile = json.compile;
                if (compile) {
                    for (var onecompile of compile) {
                        if (!util.isStringEmpty(onecompile)) {
                            compileDestFiles.push(onecompile);
                        }
                    }
                }
                var sourceMap = json.sourceMap;
                if (sourceMap) {
                    for (var onesourceMap of sourceMap) {
                        if (!util.isStringEmpty(onesourceMap)) {
                            sourceMapFiles.push(onesourceMap);
                        }
                    }
                }
                if (fs.existsSync(file)) {
                    var src = path.join(projectDir, file);
                    src = util.pathReplaceSlash(file);
                    return new ProcessorResult(projectDir, destDir, compileSrcFiles, compileDestFiles, sourceMapFiles,
                        src);
                } else {
                    if (fs.existsSync(wap2appAppidAppendCSSFile)) {
                        var toFile = path.join(destDir, serveIndexcss);
                        fs.copyFileSync(wap2appAppidAppendCSSFile, toFile);
                        compileSrcFiles.add(serveIndexcss);
                        compileDestFiles.add(serveIndexcss);
                    }
                    return ProcessorResult(projectDir, destDir, compileSrcFiles, compileDestFiles, sourceMapFiles);
                }
            }
        } catch (e) {
            if (fs.existsSync(file)) {
                var src = path.join(projectDir, file);
                src = util.pathReplaceSlash(file);
                return new ProcessorResult(projectDir, destDir, null, null, null, src);
            }
        }
        if (fs.existsSync(file)) {
            throw `文件 ${file}  转换失败，失败原因:\r\n ${outPut}`;
        }
        else{
            throw `项目 ${projectDir}  转换失败，失败原因:\r\n ${outPut}`;
        }
    }

    return ProcessorResult();
}


function processorProject(project, projectDir, destDir, file, isProduction,delCompileSrcFiles){
    if (util.isStringEmpty(project.nodePath)) return;
    if (!fs.existsSync(project.nodePath)) return;
    if(!enable(project)) return;
    var result = processFile(project, projectDir, destDir, file, isProduction);
    if(delCompileSrcFiles && result && result.compileSrcFiles){
        for (var oneSrccompile of result.compileSrcFiles) {
            if(!util.isStringEmpty(oneSrccompile)){
                var deleteFile = '';
                deleteFile = path.join(destDir,oneSrccompile);
                if(fs.existsSync(deleteFile)){
                    fs.unlinkSync(deleteFile)
                }
            }
        }
    }
    return result;
}

module.exports = {
    processorProject: processorProject
}