var fs = require("fs")
var os = require("os")
var path = require("path");
var util = require("./util")
var projectutil = require("./projectutil")
var projectprocessors = require("./projectprocessors")
let skipFiles = ["unpackage","node_modules","nativeplugins"];

async function copyPackAgeResources(project, needConfusion,checkError,sourceFile, targetDir, appconfig)
{
    sourceFile = util.pathReplaceSlash(sourceFile);
    var unpackageFiles = {};
    for(var name in skipFiles)
    {
        unpackageFiles[sourceFile + "/" + skipFiles[name]] = {};
    }
    
    try{
        if(appconfig.hasOwnProperty("unpackage"))
        {
            var array = appconfig.unpackage;
            if(array && Array.isArray(array) && array.length > 0)
            {
                for(var file in array)
                {
                    unpackageFiles[sourceFile + "/" + array[file]] = {};
                }
            }
        }
    }catch(e){
        console.error(e);
    }
    
    let copyResult = util.copyDirectory(sourceFile,targetDir,(filePath)=>{ 
        filePath = util.pathReplaceSlash(filePath);
        var filename  = path.basename(filePath);
         // App打包时，项目中文件名不能含有中文或全角字符，请确认。
        if( checkError && util.checkChinese(filename)){
            throw `App打包时，项目中文件名不能含有中文或全角字符，请确认。\r\n文件路径：${filePath}`;
        }
        if(filePath == sourceFile + "/appstream.json" || (filePath.startsWith(sourceFile +"/appstream@") && filePath.endsWith(".bak.json"))) return false;
        if(unpackageFiles.hasOwnProperty(filePath)) return false;
        if("manifest.json.encrypted" == filename.toLowerCase()) return false;
        if(filename.startsWith(".")) return false;
        return true;
    });
    
    if(!projectutil.isUniAppVue(project))
    {
        var compilePath = projectutil.getCompilePath(project);
        if(!util.isStringEmpty(compilePath) && fs.existsSync(compilePath) && fs.statSync(compilePath).isDirectory()){
            let result = util.copyDirectory(compilePath,targetDir,(filePath)=>{
                filePath = util.pathReplaceSlash(filePath);
                var filename  = path.basename(filePath);
                 // App打包时，项目中文件名不能含有中文或全角字符，请确认。
                if( checkError && util.checkChinese(filename)){
                    throw `App打包时，项目中文件名不能含有中文或全角字符，请确认。\r\n文件路径：${filePath}`;
                }
                if("manifest.json.encrypted" == filename.toLowerCase()) return false;
                if(filename.startsWith(".")) return false;
                return true;
            });
        }
        projectprocessors.processorProject(project,targetDir,targetDir,null,true,true);
    }
    
    return targetDir;
}

module.exports = {
    copyPackAgeResources: copyPackAgeResources
}