const fs = require("fs")
const path = require("path")
const archiver = require('archiver')

function isStringEmpty(obj) {
    if (typeof obj == "undefined" || obj == null || obj == "") {
        return true;
    }
    return false;
}

function getFileExists(projectPath, filePath) {
    if (isStringEmpty(filePath)) return;
    if (fs.existsSync(path.join(projectPath,filePath))) {
        return path.join(projectPath,filePath);
    }else if (fs.existsSync(filePath)) {
        return filePath;
    }
    return;
}

function copyDirectory(sourcePath, targetPath, filter) {
    if (!fs.existsSync(sourcePath)) {
        return;
    }
    if (!fs.existsSync(targetPath)) {
        fs.mkdirSync(targetPath, {
            recursive: true
        })
    }

    fs.readdirSync(sourcePath).forEach(item => {
        let absSourcePath = sourcePath + "/" + item;
        let absTargetPath = targetPath + "/" + item;
        if (fs.statSync(absSourcePath).isFile()) {
            if (filter) {
                if (filter(absSourcePath) == true) {
                    fs.copyFileSync(absSourcePath, absTargetPath)
                }
            } else {
                return fs.copyFileSync(absSourcePath, absTargetPath)
            }
        } else {
            if (filter) {
                if (filter(absSourcePath) == true) {
                    copyDirectory(absSourcePath, absTargetPath, filter);
                }
            } else {
                copyDirectory(absSourcePath, absTargetPath, filter);
            }
        }
    });
}

function CheckChinese(val) {
    var reg = new RegExp("[\\u4E00-\\u9FFF]+", "g");
    if (reg.test(val)) {
        return true;
    }
    return false;
}

function pathReplaceSlash(pathvalue) {
    var replacePath = pathvalue;
    if (replacePath) {
        replacePath = replacePath.replace(/\\/g, '/');
    }
    return replacePath;
}

function copyFile(projectPath, src, destName, destDir) {
    if (isStringEmpty(src)) return;
    var absSourcePath;
    if (fs.existsSync(projectPath + "/" + src)) {
        absSourcePath = projectPath + "/" + src;
    } else if (fs.existsSync(src)) {
        absSourcePath = src;
    }
    if (!absSourcePath) return;
    var extname = path.extname(absSourcePath);
    if(absSourcePath.endsWith(".9.png")){
        extname = ".9.png";
    }
    
    var fileName  = destName + extname;
    fs.copyFileSync(absSourcePath, path.join(destDir,fileName));

    return fileName;
}

function isEmptyObject(obj) {
    if (!obj) return true;
    for (var key in obj) {
        return false;
    };
    return true;
}

async function zipRecource(dirPath,wgtfile){
    
    try{
        if (fs.existsSync(wgtfile)){
            fs.rmdirSync(wgtfile, {
                    force: true,
                    recursive: true,
            });
        }
    }
    catch(e){
        
    }
    
    const zip = archiver('zip')
    zip.glob('**/*', {
        cwd: dirPath,
        dot:true
    }, {});
    
    var output = fs.createWriteStream(path.resolve(__dirname, wgtfile));
    zip.pipe(output);    
    zip.finalize();
    
    return new Promise((resolve,reject) => {
      output.on('close', function(){
          resolve()
      });
      
      output.on('end', function(){
         reject(new Error('zip error'))
      });
      output.on('warning', function(err){
          reject(new Error('zip error'))
      });
      output.on('error', function(err){
          reject(new Error('zip error'))
      });
  })
}
module.exports = {
    isStringEmpty: isStringEmpty,
    isEmptyObject: isEmptyObject,
    copyDirectory: copyDirectory,
    checkChinese: CheckChinese,
    pathReplaceSlash: pathReplaceSlash,
    copyFile: copyFile,
    getFileExists:getFileExists,
    zipRecource:zipRecource
}